use log::{Record, Level, Metadata, SetLoggerError};
use std::sync::{Arc, Mutex};
use chrono::Local;
use lazy_static::lazy_static;

type LogCallback = Box<dyn Fn(&str) + Send + Sync>;

pub struct FltkLogger {
    callback: Arc<Mutex<Option<LogCallback>>>,
}

impl FltkLogger {
    pub fn new() -> Self {
        FltkLogger {
            callback: Arc::new(Mutex::new(None)),
        }
    }

    pub fn set_callback<F>(&self, callback: F)
    where
        F: Fn(&str) + 'static + Send + Sync,
    {
        let mut cb = self.callback.lock().unwrap();
        *cb = Some(Box::new(callback));
    }
}

impl log::Log for FltkLogger {
    fn enabled(&self, metadata: &Metadata) -> bool {
        metadata.level() <= Level::Info
    }

    fn log(&self, record: &Record) {
        if self.enabled(record.metadata()) {
            if let Some(callback) = self.callback.lock().unwrap().as_ref() {
                callback(&format!("{}: {} - {}\r\n",Local::now().format("%Y-%m-%d %H:%M:%S"), record.level(), record.args()));
            }
        }
    }

    fn flush(&self) {}
}

lazy_static! {
   pub static ref LOGGER: FltkLogger = FltkLogger::new();
}

pub fn init_logger() -> Result<(), SetLoggerError> {
    log::set_logger(&*LOGGER)
        .map(|()| log::set_max_level(log::LevelFilter::Info))
}
