use std::cell::RefCell;
use std::rc::Rc;
use std::sync::{Arc, Mutex};
use fltk::prelude::{DisplayExt, InputExt, WidgetBase, WidgetExt};
use fltk::text::TextBuffer;
use crate::services::work_service::WorkService;
use crate::ui::main_ui;
use crate::ui::main_ui::*;
use crate::services::log_service::LOGGER;


pub(crate) struct UiService {
    ui: Rc<RefCell<UserInterface>>,
    work_service: Rc<RefCell<WorkService>>,
    current_com: String,
}

impl UiService {
    pub(crate) fn new() -> Self {
        let ui = Rc::new(RefCell::new(main_ui::UserInterface::make_window()));
        UiService { ui, work_service: Rc::new(RefCell::new(WorkService::new())), current_com: "".to_string() }
    }

    pub(crate) fn init_components(&mut self) {
        self.init_excel();
        self.init_start_btn();
        self.init_set_line_btn();
        self.init_log();
    }

    fn init_set_line_btn(&mut self) {
        let ui_rc = Rc::clone(&self.ui);
        let ui_rc_clone = Rc::clone(&ui_rc);
        let work_service_rc = Rc::clone(&self.work_service);
        ui_rc.borrow_mut().set_lines_btn.set_callback(move |_| {
            let mut work_service = work_service_rc.borrow_mut();
            let mut ui_clone = ui_rc_clone.clone();
            let mut set_dialog = UserInterface1::make_set_line_window();
            let work_service_rc_clone = work_service_rc.clone();
            set_dialog.ok_btn.set_callback(move |_| {
                if let Ok(line) = set_dialog.line_textbox.value().parse::<usize>() {
                    match work_service_rc_clone.borrow_mut().set_current_line(line) {
                        Ok(_) => {
                            ui_clone.borrow_mut().current_line_textbox.set_value(&line.to_string())
                        }
                        Err(msg) => fltk::dialog::message_default(&msg)
                    }
                } else {
                    fltk::dialog::message_default(&"Only Support Number")
                }
            })
        });
    }

    fn init_start_btn(&mut self) {
        let ui_rc = Rc::clone(&self.ui);
        let ui_rc_clone = Rc::clone(&ui_rc);
        let work_service_rc = Rc::clone(&self.work_service);
        ui_rc.borrow_mut().start_btn.set_callback(move |_| {
            let mut ui = ui_rc_clone.borrow_mut();
            let mut work_service = work_service_rc.borrow_mut();
            if let Some(mut buffer) = ui.log_content.buffer() {
                buffer.set_text("");
            }
            match work_service.excel_service {
                None => {
                    fltk::dialog::message_default("Please Select Excel File First;");
                    return;
                }
                Some(_) => {}
            }
            let selected_com = ui.com_textbox.value();
            if let Err(msg) = work_service.init_serial_service(selected_com) {
                fltk::dialog::message_default(&msg);
                return;
            }
            if let Some(mut model) = work_service.get_next_work_model() {
                ui.current_write_model.set_value(&format!("{}: {}_{}", &model.line_number, &model.imei, &model.sn));
                match work_service.write_and_check(model) {
                    Ok(_) => {
                        let current_line = &work_service.get_current_line().to_string();
                        ui.current_line_textbox.set_value(current_line);
                    }
                    Err(msg) => { fltk::dialog::message_default(&msg) }
                }
            }
        });
    }

    fn init_log(&mut self) {
        let ui_rc = Rc::clone(&self.ui);
        let text_buffer = TextBuffer::default();
        ui_rc.borrow_mut().log_content.set_buffer(text_buffer.clone());
        let text_buffer_arc = Arc::new(Mutex::new(text_buffer));
        LOGGER.set_callback(move |msg| {
            let mut text_buffer = text_buffer_arc.lock().unwrap();
            text_buffer.append(&msg);
        });
    }

    fn init_excel(&mut self) {
        let ui_rc = Rc::clone(&self.ui);
        let ui_clone = Rc::clone(&self.ui);
        let work_service_clone = Rc::clone(&self.work_service);
        ui_rc.borrow_mut().select_file_btn.set_callback(move |_| {
            let mut ui = ui_clone.borrow_mut();
            let mut work_service = work_service_clone.borrow_mut();
            let mut dialog = fltk::dialog::FileDialog::new(fltk::dialog::FileDialogType::BrowseFile);
            dialog.show();
            let binding = dialog.filename();
            let path = binding.to_str().unwrap();
            let current_line = &work_service.get_current_line().to_string();
            work_service.init_excel_service(path.to_string()).unwrap();
            ui.excel_file_textbox.set_value(path);
            ui.current_line_textbox.set_value(current_line);
        });
    }

    fn init_serial_service(&mut self) {
        let port_name = self.ui.borrow().com_textbox.value();
        match self.work_service.borrow_mut().init_serial_service(port_name) {
            Ok(_) => {}
            Err(_) => { fltk::dialog::message_default("Cant Open serial") }
        }
    }
}