use std::ptr::null;
use log::Log;
use crate::models::work_model::WorkModel;
use crate::services::{serial_service::SerialService, sqlite_service::SqliteService, excel_service::ExcelService};
use crate::services::log_service::{FltkLogger, init_logger, LOGGER};

pub struct WorkService {
    pub(crate) serial_service: Option<SerialService>,
    sqlite_service: SqliteService,
    pub(crate) excel_service: Option<ExcelService>,
}

impl WorkService {
    pub fn new() -> Self {
        init_logger().unwrap();
        WorkService {
            serial_service: Option::None,
            sqlite_service: SqliteService::new("data.db".to_string()).unwrap(),
            excel_service: Option::None,
        }
    }
    pub fn get_port_names() -> Vec<String> {
        //maybe should not use unwrap
        return SerialService::get_ports().unwrap();
    }

    pub fn init_excel_service(&mut self, path: String) -> Result<bool, String> {
        return if let Ok(excel_service) = ExcelService::new(&path) {
            self.excel_service = Some(excel_service);
            Ok(true)
        } else {
            Err("Cant Init ExcelService".to_string())
        };
    }

    pub fn write_and_check(&mut self, model: WorkModel) -> Result<bool, String> {
        let msg = &format!("{}:{}:{}", model.imei, model.sn, model.line_number);
        log::info!("Write Start");
        if let Err(msg) = self.write_imei(model.imei.clone()) {
            log::error!("Write Imei Error:{}", msg);
            return Err(msg);
        }
        log::info!("Write Imei:{}", model.imei);
        if let Err(msg) = self.write_sn(model.sn.clone()) {
            log::error!("Write Sn Error:{}", msg);
            return Err(msg);
        }
        log::info!("Write End");
        log::info!("Check Start");
        if let Err(msg) = self.check_imei(&model.imei) {
            log::error!("Check Imei Error:{}", msg);
            return Err(msg);
        }
        log::info!("Check Imei:{}", model.imei);
        if let Err(msg) = self.check_sn(&model.sn) {
            log::error!("Check Sn Error:{}", msg);
            return Err(msg);
        }
        log::info!("Check End");
        self.serial_service = None;
        Ok(true)
    }

    pub fn get_current_line(&mut self) -> usize {
        if let Some(service) = &mut self.excel_service {
            return service.get_current_line();
        }
        return 0;
    }

    pub fn set_current_line(&mut self, line: usize)->Result<(), String>{
        if let Some(service) = &mut self.excel_service{
            return service.set_current_line(line);
        }else{
            Err("Cant Set Line".to_string())
        }

    }

    pub fn get_next_work_model(&mut self) -> Option<WorkModel> {
        if let Some(service) = &mut self.excel_service {
            if let Ok(model) = service.get_next_work_model() {
                return Some(model);
            } else { return None; }
        }
        return None;
    }

    pub fn init_sqlite_service(&mut self, db_path: String) -> Result<bool, String> {
        return if let Ok(sqlite_service) = SqliteService::new(db_path) {
            self.sqlite_service = sqlite_service;
            Ok(true)
        } else {
            Err("Cant Init SqliteService".to_string())
        };
    }

    pub fn init_serial_service(&mut self, port_name: String) -> Result<bool, String> {
        return if let Ok(serial_service) = SerialService::new(&port_name) {
            self.serial_service = Some(*serial_service);
            Ok(true)
        } else {
            Err("Cant Init SerialService".to_string())
        };
    }

    fn write_imei(&mut self, imei: String) -> Result<bool, String> {
        let write_imei_command = format!("AT+ECCGSN=\"IMEI\",\"{}\"", imei);
        if let Some(service) = &mut self.serial_service {
            return service.send_command_with_target(write_imei_command, None, "OK".to_string());
        }
        return Err("Cant Init SerialService".to_string());
    }
    fn write_sn(&mut self, sn: String) -> Result<bool, String> {
        let write_sn_command = format!("AT+ECCGSN=\"SN\",\"{}\"", sn);
        if let Some(service) = &mut self.serial_service {
            return service.send_command_with_target(write_sn_command, None, "OK".to_string());
        }
        return Err("Cant Init SerialService".to_string());
    }

    fn check_imei(&mut self, imei: &str) -> Result<bool, String> {
        let check_imei_command = "AT+CGSN=1".to_string();
        if let Some(service) = &mut self.serial_service {
            return service.send_command_with_target(check_imei_command, None, imei.to_string());
        }
        return Err("Cant Init SerialService".to_string());
    }

    fn check_sn(&mut self, sn: &str) -> Result<bool, String> {
        let check_sn_command = "AT+CGSN=0".to_string();
        if let Some(service) = &mut self.serial_service {
            return service.send_command_with_target(check_sn_command, None, sn.to_string());
        }
        return Err("Cant Init ServialService".to_string());
    }
}